﻿using Microsoft.Phone.BackgroundAudio;
using Petzold.MusicSynthesis;

namespace AudioStreamAgent
{
    /// <summary>
    /// A background agent that performs per-track streaming for playback
    /// </summary>
    public class AudioTrackStreamer : AudioStreamingAgent
    {
        const int SAMPLE_RATE = 44100;

        /// <summary>
        /// Called when a new track requires audio decoding
        /// (typically because it is about to start playing)
        /// </summary>
        /// <param name="track">
        /// The track that needs audio streaming
        /// </param>
        /// <param name="streamer">
        /// The AudioStreamer object to which a MediaStreamSource should be
        /// attached to commence playback
        /// </param>
        /// <remarks>
        /// To invoke this method for a track set the Source parameter of the AudioTrack to null
        /// before setting  into the Track property of the BackgroundAudioPlayer instance
        /// property set to true;
        /// otherwise it is assumed that the system will perform all streaming
        /// and decoding
        /// </remarks>
        protected override void OnBeginStreaming(AudioTrack track, AudioStreamer streamer)
        {
            // Create a Rambler
            Rambler rambler = new Rambler(SAMPLE_RATE, 
                                            new Pitch(Note.Csharp, 4), // Start
                                            new Pitch(Note.Csharp, 2), // Minimum
                                            new Pitch(Note.Csharp, 6)) // Maximum
            {
                Oscillator = new AlmostSquareWave(SAMPLE_RATE),
                Tempo = 480
            };

            // Set allowable note values
            rambler.Notes.Add(Note.Csharp);
            rambler.Notes.Add(Note.Dsharp);
            rambler.Notes.Add(Note.Fsharp);
            rambler.Notes.Add(Note.Gsharp);
            rambler.Notes.Add(Note.Asharp);

            // Create Mixer and MixerInput objects
            Mixer mixer = new Mixer();
            mixer.Inputs.Add(new MixerInput(rambler));

            DynamicPcmStreamSource audioStreamSource = new DynamicPcmStreamSource(SAMPLE_RATE);
            audioStreamSource.SampleProvider = mixer;
            streamer.SetSource(audioStreamSource);
        }

        /// <summary>
        /// Called when the agent request is getting cancelled
        /// The call to base.OnCancel() is necessary to release the background streaming resources
        /// </summary>
        protected override void OnCancel()
        {
            base.OnCancel();
        }
    }
}
