﻿using System;
using System.Collections.Generic;

namespace Petzold.MusicSynthesis
{
    public class Rambler : SteadyNoteDurationPlayer
    {
        Random random = new Random();
        Pitch currentPitch;

        public Rambler(int sampleRate, Pitch start, Pitch lowerLimit, Pitch upperLimit)
            : base(sampleRate)
        {
            this.Notes = new List<Note>();
            this.Start = start;
            this.LowerLimit = lowerLimit;
            this.UpperLimit = upperLimit;

            currentPitch = this.Start;
        }

        public IList<Note> Notes { protected set; get; }

        public Pitch Start { protected set; get; }

        public Pitch LowerLimit { protected set; get; }

        public Pitch UpperLimit { protected set; get; }

        protected override Pitch GetNextPitch()
        {
            if (this.Notes.Count == 0)
                return new Pitch();

            int noteIndex = this.Notes.IndexOf(currentPitch.Note);
            int octave = currentPitch.Octave;
            bool goUp = false;

            if (currentPitch.Equals(this.UpperLimit))
                goUp = false;
            else if (currentPitch.Equals(this.LowerLimit))
                goUp = true;
            else
                goUp = random.Next(2) == 0;

            if (goUp)
            {
                if (noteIndex == this.Notes.Count - 1)
                    currentPitch = new Pitch(this.Notes[0], octave + 1);
                else
                    currentPitch = new Pitch(this.Notes[noteIndex + 1], octave);
            }
            else
            {
                if (noteIndex == 0)
                    currentPitch = new Pitch(this.Notes[this.Notes.Count - 1], octave - 1);
                else
                    currentPitch = new Pitch(this.Notes[noteIndex - 1], octave);
            }
            return currentPitch;
        }
    }
}
