﻿using System;

namespace Petzold.MusicSynthesis
{
    public abstract class SteadyNoteDurationPlayer : IMonoSampleProvider
    {
        int sampleRate;
        TimeSpan noteInterval;
        TimeSpan currentNoteDuration;
        Pitch pitch;

        private SteadyNoteDurationPlayer()
        {
        }

        protected SteadyNoteDurationPlayer(int sampleRate)
        {
            this.sampleRate = sampleRate;
            this.Tempo = 60;
        }

        public Oscillator Oscillator { set; get; }

        // In quarter notes per minute
        public int Tempo 
        {
            set { noteInterval = TimeSpan.FromMinutes(1.0 / value); }
            get { return (int)(1 / noteInterval.TotalMinutes); }
        }

        public short GetNextSample()
        {
            if (this.Oscillator == null)
                return 0;

            // First time through or note finished playing
            if (pitch.Frequency == 0 || currentNoteDuration <= TimeSpan.Zero)
            {
                pitch = GetNextPitch();
                this.Oscillator.Frequency = pitch.Frequency;
                currentNoteDuration = noteInterval;
            }

            currentNoteDuration -= TimeSpan.FromTicks((long)(10000000.0 / sampleRate));
            return this.Oscillator.GetNextSample();
        }

        protected abstract Pitch GetNextPitch();
    }
}
