﻿using System;
using System.Windows;
using System.Collections.Generic;

namespace Petzold.MusicSynthesis
{
    public class VariableWaveformOscillator : Oscillator
    {
        public VariableWaveformOscillator(int sampleRate)
            : base(sampleRate)
        {
            this.Points = new List<Point>();
        }

        public IList<Point> Points { set; get; }

        public override short GetNextSample(double angle)
        {
            if (this.Points == null || this.Points.Count == 0)
                return 0;

            // Assumes Points are ordered by angle (X) and evenly distributed w/o first and last
            int index2 = (int)((this.Points.Count + 1) * angle / (2 * Math.PI));
            int index1 = index2 - 1;

            Point point1 = index1 == -1 ? new Point(0, 0) : this.Points[index1];
            Point point2 = index2 == this.Points.Count ? new Point(2 * Math.PI, 0) : this.Points[index2];

            return Interpolate(angle, point1, point2);
        }

        short Interpolate(double angle, Point point1, Point point2)
        {
            return (short)(short.MaxValue * (point1.Y * (point2.X - angle) + 
                                             point2.Y * (angle - point1.X)) / (point2.X - point1.X));
        }
    }
}
